/*  osdchat - on-screen display messaging service
 *  Copyright (c) 2003 by Mike Smelov  <mike-at-argoncorp-dot-com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  See LICENSE for details.
 *
 */

 /*  cs.c */

#include "osdchatd.h"
#include "common.h"

struct client client_list[MAX_NCLIENTS];	/* master list of all clients */

/* defined in config.c */
extern struct pref_s pref;


/* initialize client list */
void cs_init(void)
{
	int i;

	for (i = 0; i < pref.max_clients; i++) {
		client_list[i].id = -1;
		client_list[i].fd = -1;
	}
}

/* return prompt associated with a client fd */
char *cs_get_prompt(int fd)
{
	int i;

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].fd == fd)
			return client_list[i].prompt;
	}

	errno = EBADF;
	return NULL;
}

/* set prompt associated with a client fd */
int cs_set_prompt(int fd, char *prompt)
{
	int i, len;
	char *tmp;

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].fd == fd) {
			len = strlen(prompt)+1;
			tmp = (char *) xmalloc(len);
			strncpy(tmp, prompt, len);
			client_list[i].prompt = tmp;
			return 1;
		}
	}

	errno = EBADF;
	return -1;
}


/* add client to the client list */
int cs_add_client(int fd, struct sockaddr_in *addr)
{
	int i = 0;
	int ip = addr->sin_addr.s_addr;

	for (i = 0; i < pref.max_clients; i++) {
		if (client_list[i].id == -1) {
			client_list[i].id = i;
			client_list[i].fd = fd;
			client_list[i].ip = ip;
			client_list[i].tm_lastmsg = 0;	/* not set until prompt is read */
			client_list[i].warning = 0;
			client_list[i].prompt = NULL;
			return 1;
		}
	}

	errno = EBADF;
	return -1;
}

/* remove client from client list & free mem used by its prompt */
int cs_del_client(int fd)
{
	int i;

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].fd == fd) {
			client_list[i].id = -1;
			client_list[i].fd = -1;
			free(client_list[i].prompt);
			return 1;
		}
	}

	errno = EBADF;
	return -1;
}

/* tell all clients that the server is going down, free mem used
   by their prompts, and close all client connections.
*/
void cs_del_close_all(void)
{
	int i, fd;
	msg_t msg = MSG_HOSTDIED;

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].id != -1) {
			fd = client_list[i].fd;

			/* tell client that the host died */
			if ( send(fd, &msg, sizeof(msg), MSG_DONTWAIT) != sizeof(msg) )
				printDBG(LVL_SYSLOG, "Warning: Client not notifed of disconnection");

			cs_del_client(fd);
			(void) close(fd);
		}
	}
}

/* set time of last received client message */
int cs_set_tm(int fd, time_t t)
{
	int i;

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].fd == fd) {
			client_list[i].tm_lastmsg = t;
			return 1;
		}
	}

	errno = EBADF;
	return -1;
}

/* get time of last received client message */
time_t cs_get_tm(int fd)
{
	int i;

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].fd == fd) {
			return client_list[i].tm_lastmsg;
		}
	}

	errno = EBADF;
	return -1;
}

/* adjust warning level and return new level */
int cs_adj_warning(int fd, int adj)
{
	int i;

	for (i = 0; i < num_clients; i++) {
		if (client_list[i].fd == fd) {
			client_list[i].warning += adj;
			return client_list[i].warning;
		}
	}

	errno = EBADF;
	return -1;
}
